function [avgReward, avgSe] = greedySensingImperfect(numberOfChannels, channelBandwidth, ...
    prob0to1, prob1to1, T, collisionProb)

% File: greedySensingImperfect.m
% Author: Sarah Chen 
% Nov. 2007
% Inputs: number of channels, bandwidth and transition probabilities of
%         each channel, time horizon T, and the maximum allowable collision probability.
% Output: average throughput of the secondary user bits/slot
%         and spectrum efficiency bits/slot/hz in the presence of sensing
%         errors
% Function: calculate the throughput performance of the greedy sensing
%           strategy in independent channels.


MAX = 1e4;
if(length(channelBandwidth) == 1)   % idential channel bandwidth
    channelBandwidth = channelBandwidth * ones(1, numberOfChannels);
end
if(length(prob0to1) == 1)   % identical channel transition probabilities
    prob0to1 = prob0to1 * ones(1, numberOfChannels);
    prob1to1 = prob1to1 * ones(1, numberOfChannels);
end
% stationary distribution of channel occupancy states
initialBelief = prob0to1 ./ ( 1 + prob0to1 - prob1to1 );    
% sensing errors
missDetection = collisionProb;
falseAlarm = solveFalseAlarm(5, missDetection, 10^(0.5)) ; % numberOfSamples = 5; snr = 5dB

reward = 0;
primary = 0;
for i = 1 : MAX
    uniformRV = rand( T + 1, numberOfChannels );    % uniform random variables
    belief = initialBelief;
    % generate intial channel occupancy states according to the initial
    % belief vector: "0" = busy, "1" = idle.
    channelState = ( uniformRV(1,:) < initialBelief );   
    for t = 1 : T
        % primary throughput 
        primary = primary + sum((1 - channelState) .* channelBandwidth);
        % Choose greedy sensing action
        idleProb = belief .* prob1to1 + (1 - belief) .* prob0to1;  
        [tmp, sensingAction] = max( idleProb .* channelBandwidth ); 
        % Observe sensing outcome, which determines access decision
        if( channelState(sensingAction) == 0 )
            sensingOutcome = (rand(1) < missDetection);
        else
            sensingOutcome = (rand(1) < 1 - falseAlarm);
        end
        % Accumulate reward
        reward = reward + sensingOutcome * channelState(sensingAction) * channelBandwidth(sensingAction);    
        primary = primary - sensingOutcome * (1 - channelState(sensingAction)) * channelBandwidth(sensingAction); 
        % Update belief vector
        belief = idleProb;
        if( sensingOutcome )
            belief(sensingAction) = 1;
        else
            belief(sensingAction) = falseAlarm * idleProb(sensingAction) ...
                / (falseAlarm * idleProb(sensingAction) + 1 - idleProb(sensingAction)) ;
        end
        % Channel state transition, 1->1 according to prob1to1 and 0->1
        % according to prob0to1
        channelState = channelState .* ( uniformRV(t+1, :) < prob1to1 ) ...
            + (1 - channelState) .* ( uniformRV(t+1, :) < prob0to1 );
    end
end
avgReward = reward / MAX / T;
avgSe = (primary + reward) / MAX / T / sum(channelBandwidth);
        
    
    