function avgReward = multiUserRandom(pktArrivalRate)

% File: multiUserRandom.m
% Author: Sarah Chen 
% Nov. 2007
% Inputs: packet arrival rate lambda of secondary users (Possion distributed) 
%         Pr(Packets = n) = lambda^n exp(-lambda) / n!
% Output: average expected accumulated reward per slot
% Function: calculate the throughput performance of competing secondary users
%           using the random sensing strategy with perfect spectrum identification 
%           in independent channels.
numberOfChannels = 10;
prob0to1 = 0.2 * ones(1, numberOfChannels);
prob1to1 = 0.8 * ones(1, numberOfChannels);
channelBandwidth = ones(1, numberOfChannels);
T = 1e3;    % time horizon
numberOfUsers = 3;
% The message length L follows a Geometrical distribution with parameter p: 
% Pr(L = n) = p (1 - p)^(n-1) avg pkt length = 50
msgLenP = 0.02; 
MAX = 1e3;

% stationary distribution of channel occupancy states
initialBelief = prob0to1 ./ ( 1 + prob0to1 - prob1to1 );    
reward = 0;

for k = 1 : MAX
    uniformRV = rand( T + 1, numberOfChannels );    % uniform random variables
    channelState = ( uniformRV(1,:) < initialBelief );   
    buffer = zeros(1, numberOfUsers); % number of pkts waiting for the transmission
    belief = ones(numberOfUsers, 1) * initialBelief;    % belief vectors of users
    
    for t = 1 : T 
        % New messages arrive in slot t-1. The number is Poisson
        % distributed with parameter pktArrivalRate.
        numberOfNewMsgs = random('Poisson', pktArrivalRate, 1, 1); 
        
        % Update users' buffer states
        if(numberOfNewMsgs > 0)   % if new message arrives
            % each new message has a geometrical distributed length in slots. Since
            % message length is positive, we need to add 1 to the generated RV.
            newMsgLen = random('Geometric', msgLenP, 1, numberOfNewMsgs) + 1;         
            % assign each new message to a secondary user uniformly  
            newMsgUser = ceil(rand(1,numberOfNewMsgs) * numberOfUsers);             
            % update the buffer of the secondary users according to the new message 
            % assignment
            buffer(newMsgUser) = buffer(newMsgUser) + newMsgLen;
        end
        
        % Choose sensing action randomly: "0" = no packet to transmit; 
        % update belief according to sensing outcomes   
        for i = 1 : numberOfUsers
            idleProb = belief(i,:) .* prob1to1 + (1 - belief(i,:)) .* prob0to1;
            if( buffer(i) == 0 )
                sensingAction(i) = 0;
                belief(i,:) = idleProb;
            else
                sensingAction(i) = ceil( rand(1) * numberOfChannels ); 
                belief(i,:) = idleProb;
                belief(i, sensingAction(i)) = channelState(sensingAction(i));
            end
        end
        
        % Resolve collisions among competing secondary users; and obtain
        % rewards
        for i = 1 : numberOfUsers
            if( sensingAction(i) > 0 && channelState(sensingAction(i))) 
                reward = reward + channelBandwidth(sensingAction(i));
                % find users with larger id that have the same sensing
                % action as user i.
                competingUsers = i + find( sensingAction(i+1:end) == sensingAction(i)); 
                if( isempty(competingUsers) )   % no competing users
                    buffer(i) = buffer(i) - channelBandwidth(sensingAction(i));
                else    % otherwise choose one user randomly for transmission
                    competingUsers = [i, competingUsers];
                    id = ceil( rand(1) * length(competingUsers));
                    buffer(competingUsers(id)) = buffer(competingUsers(id)) - channelBandwidth(sensingAction(i));
                end
                sensingAction(competingUsers) = 0;  % competing users' actions will not be considered later
            end
        end
        
        % Channel state transition, 1->1 according to prob1to1 and 0->1
        % according to prob0to1
        channelState = channelState .* ( uniformRV(t+1, :) < prob1to1 ) ...
            + (1 - channelState) .* ( uniformRV(t+1, :) < prob0to1 );
    end % t slot
end % monte_carlo

avgReward = reward / MAX / T;