function avgReward = optimalSensing(numberOfChannels, channelBandwidth, prob0to1, prob1to1, time, filename)

% File: optimalSensing.m
% Author: Sarah Chen 
% Nov. 2007
% Inputs: number of channels, bandwidth and transition probabilities of
%         each channel, time slots and the name of the file storing 
%         the results of pomdp-solve.c.
% Output: average maximum expected reward per slot.
% Function: calculate the throughput performance of the optimal sensing
%           strategy with perfect spectrum identification in independent
%           channels.
% This program requires the use of pomdp-solve.c (which can be downloaded from 
% http://www.cs.brown.edu/research/ai/pomdp/code/index.html) 
% to obtain the maximum expected reward of a given POMDP.

if(length(channelBandwidth) == 1)   % idential channel bandwidth
    channelBandwidth = channelBandwidth * ones(1, numberOfChannels);
end
if(length(prob0to1) == 1)   % identical channel transition probabilities
    prob0to1 = prob0to1 * ones(1, numberOfChannels);
    prob1to1 = prob1to1 * ones(1, numberOfChannels);
end
numberOfStates = 2^numberOfChannels;
% Initial belief vector ( N-dimensional )
initialBelief = prob0to1 ./ ( 1 + prob0to1 - prob1to1 );
% Convert the N-dimensional belief vector to the 2^N-dimensional
% belief vector, which is used in pomdp-solve.c
initialFullBelief = ones(1, numberOfStates);
for i = 0 : numberOfStates -1
    binaryI = dec2bin(i, numberOfChannels);
    for j = 1 : numberOfChannels
        if(binaryI(j) == '1')
            initialFullBelief(i+1) = initialFullBelief(i+1) * initialBelief(j);
        else
            initialFullBelief(i+1) = initialFullBelief(i+1) * (1-initialBelief(j));
        end
    end
end    

% Generate the required input file to pomdp-solve.c 
pomdpInputFile(prob0to1, prob1to1, channelBandwidth, strcat(filename,'.POMDP'));

% Use pomdp-solve.c to obtain the optimal alpha vectors and the
% corresponding optimal actions for a given network setup. 
% Then store these results in the folder ./pomdp-solve-result/.
% See details from C codes at
% http://www.cs.brown.edu/research/ai/pomdp/code/index.html.
% An example of command: pomdp-solve -p fig13a.POMDP -horizon 2 -o fig13a_00010

reward = zeros(1,length(time));
for k = 1 : length(time)
    % Read the optimal alpha vectors and the corresponding actions 
    % at each time t.
    binaryTime = dec2bin(time(k), 5);
    filenameAlpha = strcat('./pomdp-solve-result/',filename, '_', binaryTime, '.alpha');
    fid = fopen(filenameAlpha,'r');
    while (~feof(fid))
        action = fscanf(fid,'%d',1);
        if(~isempty(action))
           alphaVector = fscanf(fid, '%f', numberOfStates);
           tmpReward = initialFullBelief * alphaVector;
           if(reward(k) < tmpReward)
               reward(k) = tmpReward;
           end
         end
     end
     fclose(fid);     
 end
avgReward = reward ./ time;