function avgReward = randomSensing(numberOfChannels, channelBandwidth, prob0to1, prob1to1, T)

% File: randomSensing.m
% Author: Sarah Chen 
% Nov. 2007
% Inputs: number of channels, bandwidth and transition probabilities of
%         each channel, time horizon T.
% Output: average expected accumulated reward per slot
% Function: calculate the throughput performance of the random sensing
%           strategy with perfect spectrum identification in independent
%           channels.

MAX = 1e4;
reward = zeros(1, T);
if(length(channelBandwidth) == 1)   % idential channel bandwidth
    channelBandwidth = channelBandwidth * ones(1, numberOfChannels);
end
if(length(prob0to1) == 1)   % identical channel transition probabilities
    prob0to1 = prob0to1 * ones(1, numberOfChannels);
    prob1to1 = prob1to1 * ones(1, numberOfChannels);
end
% stationary distribution of channel occupancy states
initialBelief = prob0to1 ./ ( 1 + prob0to1 - prob1to1 );    

for i = 1 : MAX
    uniformRV = rand( T + 1, numberOfChannels );    % uniform random variables
    belief = initialBelief;
    % generate intial channel occupancy states according to the initial
    % belief vector: "0" = busy, "1" = idle.
    channelState = ( uniformRV(1,:) < initialBelief );   
    for t = 1 : T
        % Choose a sensing action randomly
        sensingAction = ceil(rand(1) * numberOfChannels);
        % Accumulate reward
        reward(t) = reward(t) + channelState(sensingAction) * channelBandwidth(sensingAction);    
        % Update belief vector
        belief = belief .* prob1to1 + (1 - belief) .* prob0to1; 
        belief(sensingAction) = channelState(sensingAction);
        % Channel state transition, 1->1 according to prob1to1 and 0->1
        % according to prob0to1
        channelState = channelState .* ( uniformRV(t+1, :) < prob1to1 ) ...
            + (1 - channelState) .* ( uniformRV(t+1, :) < prob0to1 );
    end
end
avgReward = cumsum(reward) / MAX ./ (1:T);
        
    
    